'''
Created on 2021. 7. 17.

@author: hahoe
'''
import sys
import linecache
import inspect

import platform

from os import path, mkdir, getpid, getppid, environ

import time
from pytz import timezone
from datetime import datetime

import logging

def logMsg(msg):
  countI = 0
  for stackItem_list in inspect.stack():
    if stackItem_list[3] != "logInfo" \
        and stackItem_list[3] != "logError" \
        and stackItem_list[3] != "logTrace" \
        and stackItem_list[3] != "logUnitTest" \
        and stackItem_list[3] != "logDebug" \
        and stackItem_list[3] != "logMsg" \
        and stackItem_list[3] != "logException" \
        and stackItem_list[3] != "logMsgForException"\
        and stackItem_list[3] != "logWarn" \
        and stackItem_list[3] != "logCritical":
      break
    #print("{}:[{}]".format(-countI, inspect.stack()[countI][3]))
    countI += 1
  
  try:
    msg = "\t{}:{}\t{}:{}:{}->{}:{}:{}\t{}".format(
                                   getppid(), 
                                   getpid(),
                                   inspect.stack()[countI+1][1][len(path.abspath("."))+1:],inspect.stack()[countI+1][2], inspect.stack()[countI+1][3],
                                   inspect.stack()[countI][1][len(path.abspath("."))+1:],inspect.stack()[countI][2], inspect.stack()[countI][3],
                                   msg)
  except:
    msg = "\t{}:{}\t{}:{}:{}\t{}".format(
                                   getppid(), 
                                   getpid(),
                                   inspect.stack()[countI][1][len(path.abspath("."))+1:],inspect.stack()[countI][2], inspect.stack()[countI][3],
                                   msg)
    
  return msg

def logMsgForException(msg = None):
  exc_type, exc_obj, tb = sys.exc_info()
  f = tb.tb_frame
  lineno = tb.tb_lineno
  filename = f.f_code.co_filename[len(path.abspath("./"))+1:]
  
  countI = 0
  for stackItem_list in inspect.stack():
    if stackItem_list[3] != "logException" and stackItem_list[3] != "logMsgForException":
      break
    #print("{}:[{}]".format(-countI, inspect.stack()[countI][3]))
    countI += 1
  
  if msg == None:
    try:
      msg = "\t{}:{}\t{}:{}:{}->{}:{}:{}\tEXCEPTION IN ({}) \"{}\"".format(
                                                    getppid(), 
                                                    getpid(),
                                                    inspect.stack()[countI+1][1][len(path.abspath("."))+1:],inspect.stack()[countI+1][2], inspect.stack()[countI+1][3],
                                                    filename, lineno, inspect.stack()[countI][3],
                                                    exc_type, exc_obj
                                                    )
    except:
      msg = "\t{}:{}\t{}:{}:{}\tEXCEPTION IN ({}) \"{}\"".format(
                                                    getppid(), 
                                                    getpid(),
                                                    filename, lineno, inspect.stack()[countI][3],
                                                    exc_type, exc_obj
                                                    )
      
  else:
    try:
      msg = "\t{}:{}\t{}:{}:{}->{}:{}:{}\tEXCEPTION IN ({}) \"{}\" -> Error:[{}]".format(
                                                  getppid(), 
                                                  getpid(),
                                                  inspect.stack()[countI+1][1][len(path.abspath("."))+1:],inspect.stack()[countI+1][2], inspect.stack()[countI+1][3],
                                                  filename, lineno, inspect.stack()[countI][3],
                                                  exc_type, exc_obj,
                                                  msg
                                                  )
    except:
      msg = "\t{}:{}\t{}:{}:{}\tEXCEPTION IN ({}) \"{}\" -> Error:[{}]".format(
                                                  getppid(), 
                                                  getpid(),
                                                  filename, lineno, inspect.stack()[countI][3],
                                                  exc_type, exc_obj,
                                                  msg
                                                  )
  return msg

def initLog(logLevel = "DEBUG"):
  if platform.system().startswith("Win"):
    logDir = path.expanduser("~\AppData\Local\Temp")
  else:
    logDir = path.expanduser("~/tmp")
  
  logMetaFilename = path.join(logDir, "graphcode-{}.log".format(getppid()))
  #print(logMsg(logMetaFilename))
    
  if path.exists(logMetaFilename):
    f = open(logMetaFilename, "r")
    logFilename = f.readline()
    f.close()
    print(logMsg("logFilename:[{}] is loaded from '{}'".format(logFilename, logMetaFilename)))
    
    thisLogMetaFilename = path.join(logDir, "graphcode-{}.log".format(getpid()))
    print(logMsg(thisLogMetaFilename))
    f = open(thisLogMetaFilename, "w")
    f.write(logFilename)
    f.close()
    print(logMsg("logFilename:[{}] is saved at '{}'".format(logFilename, thisLogMetaFilename)))
    
  else:
    print(logMsg("logMetaFilename:[{}] is not found".format(logMetaFilename)))
    
    logMetaFilename = path.join(logDir, "graphcode-{}.log".format(getpid()))
    #print(logMsg(logMetaFilename))
    
    if path.exists(logMetaFilename):
      f = open(logMetaFilename, "r")
      logFilename = f.readline()
      print(logMsg("logFilename:[{}] is loaded from '{}'".format(logFilename, logMetaFilename)))
    
    else:
      print(logMsg("logMetaFilename:[{}] is not found".format(logMetaFilename)))
      
      logFileTimestamp = datetime.fromtimestamp(time.time()).astimezone(timezone('UTC')).strftime('%Y-%m-%dT%H-%M-%S_%f%Z')
      logFilename = path.join(logDir, "graphcode-{}.log".format(logFileTimestamp))
      print(logMsg("logFilename:[{}] is set".format(logFilename)))
    
      thisLogMetaFilename = path.join(logDir, "graphcode-{}.log".format(getpid()))
      print(logMsg(thisLogMetaFilename))
      f = open(thisLogMetaFilename, "w")
      f.write(logFilename)
      f.close()
      print(logMsg("logFilename:[{}] is saved at '{}'".format(logFilename, thisLogMetaFilename)))
      
      f = open(logMetaFilename, "r")
      logFilename = f.readline()
      f.close()
      print(logMsg("logFilename:[{}] is loaded from '{}'".format(logFilename, logMetaFilename)))

  logPriorityNumber = getLogPriorityNumber(logLevel)
  
  logging.basicConfig(
    handlers=[
        logging.FileHandler(logFilename),
        logging.StreamHandler()
    ],
    format='%(asctime)s %(levelname)s %(message)s', 
    datefmt='%m/%d/%Y %H:%M:%S UTC',
    level=logPriorityNumber)
  
  logInfo("logging is initialized with filename:[{}](level:{}:{})".format(logFilename, logLevel, logging.getLogger().level))
  
  return True

def setLogLevel(logLevel):
  logPriorityNumber = getLogPriorityNumber(logLevel)
  
  logging.getLogger().setLevel(logPriorityNumber)
  
  logDebug("logLevel:[{}]({}) is set".format(logLevel, logging.getLogger().level))
  
def getLogPriorityNumber(logLevel):
  if logLevel == "DEBUG":
    logPriorityNumber = logging.DEBUG
  elif logLevel == "INFO":
    logPriorityNumber = logging.INFO
  elif logLevel == "WARN":
    logPriorityNumber = logging.WARN
  elif logLevel == "ERROR":
    logPriorityNumber = logging.ERROR
  elif logLevel == "CRITICAL":
    logPriorityNumber = logging.CRITICAL
  else:
    logPriorityNumber = logging.INFO
    
  return logPriorityNumber
    
def logDebug(msg):
  logging.debug(logMsg(msg))
  
  return msg
  
def logInfo(msg):
  logging.info(logMsg(msg))
  
  return msg
  
def logWarn(msg):
  logging.warn(logMsg(msg))
  
  return msg
  
def logError(msg):
  logging.error(logMsg(msg))
  
  return msg

def raiseValueError(msg):
  logging.error(logMsg(msg))
  raise ValueError(msg)

def logException(msg = None):
  logging.exception(logMsgForException(msg))
  
  return msg

def logExceptionWithValueError(msg = None):
  logging.exception(logMsgForException(msg))
  raise ValueError(msg)

def logCritical(msg):
  logging.critical(logMsg(msg))
  
  return msg